%------------------------------------------------------------------------
% Routine     : proptime()
%
% Description : Calculate propagation times between field_pts
%               and elem_pts with a	sound speed of c, referred
%               to the time from field_pts to origo:
%               (time to origo) - (time to elements). i.e.
%               considering reception from field point m, the signal
%               received at element n is s_m0(time + tpropag(m,n)),
%               where s_m0(time) is a signal received from point m 
%               on an element in origo. The columns of elem_pts 
%               are (x,y,z) coordinates of points in space.
%               If nearfield is nonzero, the same goes for field_pts. 
%               If nearfield is zero,
%	        the field points are considered to be located on a
%               spherical surface at infinite distance, specified by 
%               their spherical coordinate angles: a column of field_pts
%               contains (phi_z,theta_x) in degrees.
%
%               Reduced form point descriptions are allowed as follows:
%               Columns of elem_pts may be (x,y) or (x), assuming missing
%               components to be 0.
%               If nearfield is nonzero, columns of field_pts may be (x,z),
%               assuming y = 0.
%               If nearfield is zero, columns of field_pts may be (phi_z), 
%               assuming theta_x = 0.
% 	        However internal calculations are always performed with 
%               full descriptions.
%
% Language    : Matlab 4.1
% Written by  : Tor Arne Reinen, SINTEF-DELAB  April 1992 
%
% Version no. : 1.0 TAR
%               2.0 SH   14.04.93
%               2.5 FT   21.04.94 Removed normalizing of prop_times when CW
%               3.0 FT   03.05.94 Changed name protime_r0 -> proptime
%               3.1 TKL  04.05.94 Corrected protime_r0 -> proptime
%
% Called by   : bpfile3.m
% Calling     : 
%------------------------------------------------------------------------

function [tpropag,distance] = proptime(field_pts,elem_pts,c,nearfield,CW)

normalize = 0;  % If flag normalize is set the propagation times are
                % normalized to the propagation time from obs. pt. to
		% the origin.

%-------------element points, full descriptions

	[m,n_el] = size(elem_pts);
	elpts3 = zeros(3,n_el);
	for i = 1:min(m,3),
	   elpts3(i,:) = elem_pts(i,:);
	end

% -------------field points, full descriptions

	[m,n_fld] = size(field_pts);
 
	if nearfield
	   if m == 2,
	      fldpts3 = zeros(3,n_fld);
	      fldpts3(1,:) = field_pts(1,:);
	      fldpts3(3,:) = field_pts(2,:);
	   else
	      fldpts3 = field_pts;
	   end
	else
%	   ---- store in flddir3: cartesian coordinates of unit vectors in directions 

	   if m == 1,
	      flddir3 = [sin(pi/180*field_pts)
			 zeros(1,n_fld)
			 cos(pi/180*field_pts)];
	   else
	      flddir3 = [sin(pi/180*field_pts(1,:)).*cos(pi/180*field_pts(2,:))
			 sin(pi/180*field_pts(1,:)).*sin(pi/180*field_pts(2,:))
			 cos(pi/180*field_pts(1,:))];
	   end

	end
  

% --------calculate distances. 

    	if nearfield

%	   ----------calculate squared absolute distances: a*a + b*b -2*a*b*cos(a,b)


	   tpropag = (sum(fldpts3.*fldpts3))'*ones(1,n_el) ...
		+ ones(n_fld,1)*(sum(elpts3.*elpts3)) ...
		- 2*fldpts3'*elpts3;
	   
%
% distance added 7/4-93 SH: 
%
	   distance = sqrt(tpropag);

%	   -----------find distances relative to reference distance

           if ~CW
	     tpropag = (sqrt(sum(fldpts3.*fldpts3)))'*ones(1,n_el) - distance;
	   else
	     tpropag =  - distance;
	   end;
	     
	else

	   tpropag = flddir3'*elpts3;
	   distance = ones(size(tpropag));

     	end

%----------convert from distance to time

	tpropag = tpropag/c;	

	return;


