% -----------------------------------------------------------------------------
% Function    : centers=randthin(centers, N_to_thin, N_to_thin2, method,
%                                distribution, symmetry, w_SD, trsd)
%
% Description : Random thinning of array elements. 
%               N_to_thin    = no of elements to remove in 1D or
%                              dim 1 in binned 2D
%               N_to_thin    = no of el to remove in random 2D
%               N_to_thin2   = no of el to remove in dim2 in binned 2D
%               method       = 'r' random thinning
%                              'b' binned 
%               distribution = 'g' Gaussian, stddev
%                              'u' uniform
%                              't' triangular
%               symmetry     =  1  no symmetry
%                               2  Bi-symmetry e(m,n)=e(-m,-n)
%                               4  Quad-symmetry
%                                       e(m,n)=e(-m,n)=e(-m,-n)=e(m,-n)
% 
%               trsd         = 1 or 2 for 1-d or 2-d array
%
%               Binned thinning means that if 25% shall be kept from a 128
%               element array, 1 element is placed in [1..4], 1 in [5..8]
%               etc according to the probability distribution
%
%
% Language    : Matlab 4.2
% Written by  : Bjrnar Elgetun/Sverre Holm, Ifi-UiO
%
% Version no. : 1.0 BJE/SH  21.07.97 First version taken from thin.m
%                       SH  28.07.97 added trsd in input
%                       SH  10.09.97 binned 2D thinning  
%
% Called by   : thin.m
% Calling     : triang, randn, rand
% -----------------------------------------------------------------------------

function centers=randthin(centers,N_to_thin, N_to_thin2, method, distribution, ...
                          symmetry, w_SD, trsd)
 
%
% THINNING
%
  N =length(centers);
  r_elem=zeros(1,N);	% find element distance r from array center x,y,z coordinate


  for n=1:N
      r_elem(n)=norm(centers(1:3,n));
  end
  [r_elem,index_elem]=sort(r_elem);% sort elements by ascending r
  if (symmetry==1)  % Removing outer two elements. AA, 19.03.97
    r_elem=r_elem(1:N-2*symmetry); 	% remove the nearest and farthest elements  
    index_elem=index_elem(1:N-2*symmetry);
  else
    r_elem=r_elem(1+symmetry:N-symmetry); 	% remove the nearest and farthest elements
  index_elem=index_elem(1+symmetry:N-symmetry); % from thinning procedure
  end
  r_elem=r_elem/max(r_elem);	    		% normalize r such that r in [0,1]

%
% main thinning routine
%
  
%***  
if (method == 'r')  % main loop for random thinning  
%***  
  
  N_thinned=0;
  while (N_thinned < N_to_thin)

%
% draw radius corresponding to actual element
%
      if (distribution == 'u')			% uniform distribution [0,1]
	  r_thin=rand;          		% draw radius
	elseif (distribution == 't')

	  
	  r_thin=1 - 2*abs(triang-0.5);
	else					% gaussian distribution
%
% SH 11.07.97: algorithm could be faster if r_thin = abs(randn)
%
	  r_thin=randn;				% draw radius
	  while(r_thin < 0 | r_thin > w_SD)	% within actual distribution interval
	      r_thin=randn;
	  end;
	  r_thin=1-(r_thin/w_SD);		% scaled antigaussian for r in [0,1]
      end

%
% search for actual radii
%
      index_r=find(r_elem > r_thin);
      index_r=index_r(1);			% pick first
      if ~isempty(index_r)

%
% two or more equal radii - choose which one by uniform draw
%
	  equal=length(find(r_elem == r_elem(index_r)));
	  if (equal > 1)
	      offset=floor(rand*(equal-eps));	% uniform draw
	      index_r=index_r+offset;		% update index to actual radius
	  end

%
% find element corresponding to actual radius
%
	  actual=index_elem(index_r);

%
% start thinning if not already removed
%
	  if (centers(4,actual) >= 0)

%
% no symmetry
%	     
              if (symmetry == 1)
		  centers(4,actual)=-centers(4,actual);		% remove element
		  N_thinned=N_thinned+1;

%
% bi-symmetry
%
		elseif (symmetry == 2)
		  centers(4,actual)=-centers(4,actual);		% remove elements
		  centers(4,N+1-actual)=-centers(4,N+1-actual);	% mirror about origin
		  N_thinned=N_thinned+2;

%
% quad-symmetry
%
		else

%
% detect elements in the actual column (y-direction) and
% mirror about x-axis
%
                  elem_y=find(abs(centers(4,:)) == abs(centers(4,actual)));
		  N_y=length(elem_y); 	 				% Total elements in column
		  n_y=find(elem_y == actual); 				% which number in column
		  actual2=actual+N_y+1-2*n_y;				% mirror about x-axis
		  centers(4,actual)=-centers(4,actual);		% remove elements
		  centers(4,N+1-actual)=-centers(4,N+1-actual);	% mirror about origin
		  centers(4,actual2)=-centers(4,actual2);		% remove elements
		  centers(4,N+1-actual2)=-centers(4,N+1-actual2);	% mirror about origin
		  N_thinned=N_thinned+4;
	      end
	  end 		% start thinning
      end 	% if
%
% algorithm could perhaps be faster if one had updated r_elem and shortened
% it, but perhaps this would result in a different distribution?
%
end 				% while

%***  
elseif (method == 'b')  % main loop for binned random thinning  SH 11.07.97
%***  

%
% remove all elements
%
centers(4,:) = -abs(centers(4,:));


 if (trsd == 1) % 1-d transducer  
  N_bin = N/(N-N_to_thin);
  disp(['Bin size ',num2str(N_bin)]);

  if symmetry == 1,
    N_remove = N-N_to_thin;
  elseif symmetry == 2,
    N_remove =0.5*(N-N_to_thin);
  end

  if (distribution == 'u')
    for n = 1:N_remove
      bin = round(rand*N_bin + 0.5);
      actual = (n-1)*N_bin + bin;
      centers(4,actual) = -centers(4,actual);
      if symmetry == 2,
        centers(4,N+1-actual)=-centers(4,N+1-actual);	% mirror about origin
      end
    end
  elseif (distribution == 't')
    for n = 1:N_remove
      bin = round(triang*N_bin + 0.5);
      actual = (n-1)*N_bin + bin;   
      centers(4,actual) = -centers(4,actual);
      if symmetry == 2,
        centers(4,N+1-actual)=-centers(4,N+1-actual);	% mirror about origin
      end
    end 
  else 
    for n = 1:N_remove
      numb = 2;
      while abs(numb) > 1 
      	numb = randn/w_SD;
      end
      bin = round((numb+1)*N_bin/2 + 0.5);
      actual = (n-1)*N_bin + bin;
      centers(4,actual) = -centers(4,actual);
      if symmetry == 2,
        centers(4,N+1-actual)=-centers(4,N+1-actual);	% mirror about origin
      end
    end 
  end

 elseif (trsd == 2)

   if symmetry ~= 1 | distribution == 'g',
	disp('Only binned 2D asymmetric, not gaussian thinning!')
   end

   N1 = max(abs(centers(4,:)));
   N2 = max(centers(5,:));
%
% n1,n2 bin-size in x- and y-directions
%
   nrem_1 = N1 - N_to_thin;
   nrem_2 = N2 - N_to_thin2;
   n1 = N1/nrem_1;
   n2 = N1/nrem_2;
   if (rem(n1,1) == 0 & rem(n2,1) == 0),

    if (distribution == 'u')
    for n=1:nrem_1
     for k=1:nrem_2
        bin1 = round(rand*n1 + 0.5);
        bin2 = round(rand*n2 + 0.5);
        xy =[(n-1)*n1+bin1, (k-1)*n2+bin2];
        actual = find(abs(centers(4,:)) == xy(1) & centers(5,:) == xy(2));
        centers(4,actual) = -centers(4,actual);
     end
    end
   elseif (distribution == 't')    
    for n=1:nrem_1
     for k=1:nrem_2
        bin1 = round(triang*n1 + 0.5);
        bin2 = round(triang*n2 + 0.5);
        xy =[(n-1)*n1+bin1, (k-1)*n2+bin2];
        actual = find(abs(centers(4,:)) == xy(1) & centers(5,:) == xy(2));
        centers(4,actual) = -centers(4,actual);
     end
    end
   end 

   else
     disp('Non-integer bin-siz, no thinning performed!')
     centers(4,:) = abs(centers(4,:));
   end     
 end % trsd = 2
%***  
end
%***

return
